<?php

/**
 * @file
 * Provides the VirusTotal API service class.
 *
 * This is based on the guidelines of
 * @link http://www.virustotal.com/advanced.html#publicapi VirusTotal API @endlink
 */

/**
 * Request was valid and could be processed successfully.
 */
define('VIRUSTOTAL_RESULT_OK', 1);

/**
 * The searched item could not be found.
 */
define('VIRUSTOTAL_RESULT_NOT_FOUND', 0);

/**
 * Used API key was not valid.
 */
define('VIRUSTOTAL_RESULT_INVALID_KEY', -1);

/**
 * API request rate was exceeded.
 */
define('VIRUSTOTAL_RESULT_EXCEED_QUOTA', -2);

/**
 * cURL HTTP request failed.
 */
define('VIRUSTOTAL_RESULT_HTTP_ERROR', -10);

/**
 * API function names.
 */
define('VIRUSTOTAL_FUNC_SCAN_FILE', 'scan_file');
define('VIRUSTOTAL_FUNC_GET_FILE_REPORT', 'get_file_report');
define('VIRUSTOTAL_FUNC_SCAN_URL', 'scan_url');
define('VIRUSTOTAL_FUNC_GET_URL_REPORT', 'get_url_report');
define('VIRUSTOTAL_FUNC_MAKE_COMMENT', 'make_comment');

/**
 * Class for VirusTotal API.
 */
class VirusTotal {

  /**
   * VirusTotal API URL
   *
   * In order to use the VirusTotal API we need to send HTTP requests to
   * the following URL.
   *
   * @var string
   */
  protected $apiUrl = 'https://www.virustotal.com/api/';

  /**
   * The required API Key to access the VirusTotal API.
   *
   * Once you have a valid VirusTotal Community account, you will find your
   * personal API key in the inbox of your account (sign in and drop down the
   * My account menu). This key is all you need to use VirusTotal's API.
   *
   * @var string
   */
  protected $apiKey = NULL;

  /**
   * VirusTotal API return Format.
   *
   * Currently the API can only return JSON formatted data.
   *
   * @var string
   */
  protected $apiFormat = '.json';

  /**
   * Constructor for VirusTotal API class.
   *
   * @param $api_key
   *   The required API Key to access the VirusTotal API.
   */
  public function __construct($api_key = NULL) {
    // Override configurated API Key if given.
    $this->apiKey = (empty($api_key) ? variable_get('virustotal_api_key', '') : $api_key);
  }

  /**
   * Sends a File to VirusTotal service to queue it for scanning.
   *
   * The VirusTotal API allows you to send files. Before performing your
   * submissions we encourage you to retrieve the latest report on the
   * files, if it is recent enough you might want to save time and bandwidth
   * by making use of it.
   *
   * @param $fobj
   *   File Object of the local file you want to send.
   * @return
   *   An array containing the scan ID ('scan_id').
   *   It will at least have a 'result' key, containing the API status code.
   */
  public function scanFile($fobj) {

    // Get real path of file.
    $realPath = $fobj->filepath;

    // Set HTTP multipart file information, so cURL can load and send it.
    return ($this->apiRequest(VIRUSTOTAL_FUNC_SCAN_FILE,
      array('file' => '@' . $realPath . ';type=' . $fobj->filemime . ';filename=' . $fobj->filename)
    ));
  }

  /**
   * Tries to retrieve a file scan report.
   *
   * @param $resource
   *   A md5/sha1/sha256 hash will retrieve the most recent report on a
   *   given sample. You may also specify a permalink identifier
   *   (sha256-timestamp (Scan ID) as returned by the file upload API)
   *   to access a specific report.
   * @return
   *   Array representing the scan report:
   *   - 'report': Array:
   *     - 0: String containing the reports date.
   *     - 1: Array containing a list of AV results.
   *   - 'permalink': Permanent link to report page.
   *   - 'result': Result status code.
   */
  public function getFileReport($resource) {
    return ($this->apiRequest(VIRUSTOTAL_FUNC_GET_FILE_REPORT, array('resource' => $resource)));
  }

  /**
   * Sends an URL to VirusTotal service and queue it for scanning.
   *
   * URLs can also be submitted for scanning. Once again, before performing
   * your submission we encourage you to retrieve the latest report on the
   * URL, if it is recent enough you might want to save time and bandwidth
   * by making use of it.
   *
   * @param $url
   *   The string of the URL that should be scanned.
   * @return
   *   An array containing the scan ID ('scan_id').
   *   It will at least have a 'result' key, containing the API status code.
   */
  public function scanUrl($url) {
    return ($this->apiRequest(VIRUSTOTAL_FUNC_SCAN_URL, array('url' => $url)));
  }

  /**
   * Tries to retrieve a URL scan report.
   *
   * @param $resource
   *   A URL will retrieve the most recent report on the given URL. You may
   *   also specify a permalink identifier (md5-timestamp as returned by the
   *   URL submission API) to access a specific report.
   * @return
   *   Array representing the scan report:
   *   - 'report': Array:
   *     - 0: String containing the reports date.
   *     - 1: Array containing a list of AV results.
   *   - 'permalink': Permanent link to report page.
   *   - 'result': Result status code.
   */
  public function getUrlReport($resource) {
    return ($this->apiRequest(VIRUSTOTAL_FUNC_GET_URL_REPORT, array('resource' => $resource)));
  }

  /**
   * Creates a comment on a file or URL report.
   *
   * @param $type
   *   String containing 'file' or 'url' depending on what you want to comment.
   * @param $resource
   *   Either a md5/sha1/sha256 hash of the file you want to review or the URL
   *   itself that you want to comment on.
   * @param $comment
   *   The actual review, you can tag it using the "#" twitter-like syntax
   *   (e.g. #disinfection #zbot) and reference users using the "@" syntax
   *   (e.g. @EmilianoMartinez).
   * @param $tags
   *   This parameter is optional. It should be an array containing a list of
   *   standard file or URL tags.
   *   - The standard file tags are: goodware, malware, spamattachmentorlink,
   *     p2pdownload, impropagating, networkworm, drivebydownload.
   *   - The standard URL tags are: malicious, benign, malwaredownload,
   *     phishingsite, browserexploit, spamlink.
   * @return
   *   Returns an array containing a 'result' key of API status code.
   */
  public function makeComment($type = 'file', $resource, $comment, $tags = array()) {
    return ($this->apiRequest(VIRUSTOTAL_FUNC_MAKE_COMMENT,
      array($type => $resource,
        'comment' => $comment,
        'tags' => implode(';', $tags),
      ))
    );
  }

  /**
   * Sends the API request and interpretes the returned data.
   *
   * @param $function
   *   API specific function name (used in URL).
   * @param $data
   *   An array containing request data as name/value pairs.
   * @return
   *   An array containing decoded json data.
   *   It will at least have a 'result' key, containing a status code.
   */
  protected function apiRequest($function, $data = array()) {

    // Allow modules to make changes on post data.
    module_invoke_all('virustotal_query_alter', $data, $function);

    // Initialise cURL.
    $curl_handle = curl_init($this->apiUrl . $function . $this->apiFormat);

    $curl_options = array(
      // Set POST data.
      CURLOPT_POST => TRUE,
      // Add key to HTTP POST data.
      CURLOPT_POSTFIELDS => $data + array('key' => $this->apiKey),
      // Results will be returned and not displayed.
      CURLOPT_RETURNTRANSFER => TRUE,
      // Don't follow redirects (there wont be any).
      CURLOPT_FOLLOWLOCATION => FALSE,
      // Forbid reuse so no quota can be wasted.
      CURLOPT_FORBID_REUSE => TRUE,
      // This is a fix for the Curl library to prevent
      // Expect: 100-continue headers in POST requests.
      CURLOPT_HTTPHEADER => array('Expect:'),
    );

    // Set cURL options.
    curl_setopt_array($curl_handle, $curl_options);

    // Send the HTTP POST request with cURL.
    $json_data = curl_exec($curl_handle);

    // Check for cURL errors.
    if (curl_errno($curl_handle)) {
      $result_data = array('result' => VIRUSTOTAL_RESULT_HTTP_ERROR);
    }
    else {
      // Decode returned json data to an array.
      $result_data = json_decode($json_data, TRUE);
    }

    // Close cURL handle.
    curl_close($curl_handle);

    // Log the result by watchdog.
    $this->logResult($result_data['result'], $function);

    // Allow modules to react on query result.
    module_invoke_all('virustotal_result_alter', $result_data, $function);

    return $result_data;
  }

  /**
   * Logs results of API request with Drupals watchdog.
   *
   * @param $result
   *   The returned result code.
   * @param $function
   *   API specific function name.
   */
  protected function logResult($result, $function) {

    // Default values.
    $severity = WATCHDOG_NOTICE;
    $message = '';

    switch ($result) {
      case VIRUSTOTAL_RESULT_OK:
        $message = t('Request was valid and could be processed successfully.');
        break;
      case VIRUSTOTAL_RESULT_NOT_FOUND:
        $message = t('The searched item could not be found.');
        $severity = WATCHDOG_ERROR;
        break;
      case VIRUSTOTAL_RESULT_INVALID_KEY:
        $message = t('Used API key was not valid.');
        $severity = WATCHDOG_ERROR;
        break;
      case VIRUSTOTAL_RESULT_EXCEED_QUOTA:
        $message = t('API request rate was exceeded.');
        $severity = WATCHDOG_ERROR;
        break;
      case VIRUSTOTAL_RESULT_HTTP_ERROR:
        $message = t('cURL HTTP request failed.');
        $severity = WATCHDOG_ERROR;
        break;
    }

    // Add API function string and log it.
    $message .= ' ' . t('API function: @function', array('@function' => $function));
    watchdog('virustotal', $message, NULL, $severity);
  }
}
